@@ -6,17 +6,13 @@ NULL
 #' Date-time must be a POSIXct, POSIXlt, Date, Period, chron, yearmon, yearqtr, zoo, 
 #' zooreg, timeDate, xts, its, ti, jul, timeSeries, and fts objects. 
 #'
-#' @export month "month<-"
-#' @aliases month month<-
-#' @S3method month default
-#' @S3method month numeric
-#' @S3method month Period
 #' @param x a date-time object  
 #' @param label logical. TRUE will display the month as a character string
 #'   such as "January." FALSE will display the month as a number.
 #' @param abbr logical. FALSE will display the month as a character string #'
 #'   label, such as #' "January". TRUE will display an abbreviated version of
 #'   the label, such as "Jan". abbr is #' disregarded if label = FALSE.
+#' @param value a numeric object
 #' @return the months element of x as a number (1-12) or character string. 1 = January.
 #' @keywords utilities manip chron methods
 #' @examples
@@ -29,17 +25,20 @@ NULL
 #' month(ymd(080101))
 #' # 1
 #' month(ymd(080101), label = TRUE)
-#' # "January"
-#' month(ymd(080101), label = TRUE, abbr = TRUE)
 #' # "Jan"
-#' month(ymd(080101) + months(0:11), label = TRUE, abbr = TRUE)
+#' month(ymd(080101), label = TRUE, abbr = FALSE)
+#' # "January"
+#' month(ymd(080101) + months(0:11), label = TRUE)
 #' # "Jan" "Feb" "Mar" "Apr" "May" "Jun" "Jul" "Aug" "Sep" "Oct" "Nov" "Dec"
+#' @export
 month <- function(x, label = FALSE, abbr = TRUE) 
   UseMethod("month")
-  
+
+#' @export
 month.default <- function(x, label = FALSE, abbr = TRUE)
   month(as.POSIXlt(x, tz = tz(x))$mon + 1, label, abbr)
   
+#' @export
 month.numeric <- function(x, label = FALSE, abbr = TRUE) {
   if (!label) return(x)
   
@@ -55,9 +54,12 @@ month.numeric <- function(x, label = FALSE, abbr = TRUE) {
   ordered(x, levels = 1:12, labels = labels)
 }
     
+#' @export
 month.Period <- function(x, label = FALSE, abbr = TRUE)
   slot(x, "month")
 
+#' @rdname month
+#' @export
 "month<-" <- function(x, value) {
 	if (!is.numeric(value)) {
 			value <- pmatch(tolower(value), c("january", "february", "march", 
@@ -66,28 +68,6 @@ month.Period <- function(x, label = FALSE, abbr = TRUE)
   	x <- x + months(value - month(x))
  }
 
-#' Get the number of days in the month of a date-time.
-#' 
-#' Date-time must be a POSIXct, POSIXlt, Date, chron, yearmon, yearqtr, 
-#' zoo, zooreg, timeDate, xts, its, ti, jul, timeSeries, and fts objects.
-#'
-#' @export days_in_month
-#' @param x a date-time object
-#' @return An integer of the number of days in the month component of the date-time object.
-days_in_month <- function(x) {
-  month_x <- month(x, label = TRUE)
-  N_DAYS <- c(
-    Jan = 31L, Feb = 28L, Mar = 31L, 
-    Apr = 30L, May = 31L, Jun = 30L, 
-    Jul = 31L, Aug = 31L, Sep = 30L, 
-    Oct = 31L, Nov = 30L, Dec = 31L
-  )
-  n_days <- N_DAYS[month_x]
-  n_days[month_x == "Feb" & leap_year(x)] <- 29L
-  n_days
-}
-
-
 setGeneric("month<-")
 
 #' @export
@@ -95,3 +75,44 @@ setMethod("month<-", signature("Period"), function(x, value){
   slot(x, "month") <- value
   x
 })
+
+#' Get the number of days in the month of a date-time.
+#' 
+#' Date-time must be a POSIXct, POSIXlt, Date, chron, yearmon, yearqtr, 
+#' zoo, zooreg, timeDate, xts, its, ti, jul, timeSeries, and fts objects.
+#'
+#' @export
+#' @param x a date-time object
+#' @return An integer of the number of days in the month component of the date-time object.
+days_in_month <- function(x) {
+  month_x <- month(x, label = TRUE)
+  n_days <- N_DAYS_IN_MONTHS[month_x]
+  n_days[month_x == "Feb" & leap_year(x)] <- 29L
+  n_days
+}
+
+## fixme: integrate with above, this oen is needed internally
+.days_in_month <- function(m, y){
+  n_days <- N_DAYS_IN_MONTHS[m]
+  n_days[m == 2L & leap_year(y)] <- 29L
+  n_days
+}
+
+## tothink: export?
+days_in_months_so_far <- function(month, leap){
+  ## if month is negative, compute from the end of the year
+  cum_days_pos <- c(0, cumsum(N_DAYS_IN_MONTHS)[-12])
+  cum_days_neg <- c(0, cumsum(rev(N_DAYS_IN_MONTHS))[-12])
+  negative <- month < 0
+  positive <- month > 0
+  sofar <- integer(length(month))
+  sofar[negative] <- cum_days_neg[-month[negative]]
+  sofar[positive] <- cum_days_pos[month[positive]]
+  adjust <- leap & ((negative & month == -12) | (positive & month > 2)) 
+  sofar[adjust] <- sofar[adjust] + 1L
+  sofar
+}
+## days_in_months_so_far(c(1, 2, 3, -10, -11, -12), rep.int(T, 6))
+## [1]   0  31  60 275 306 335
+## days_in_months_so_far(c(1, 2, 3, -10, -11, -12), rep.int(F, 6))
+## [1]   0  31  59 275 306 334
