@@ -25,8 +25,6 @@ add_duration_to_date <- function(dur, date) {
   reclass_date(new, date)
 }
 
-
-
 add_period_to_period <- function(per2, per1){
   new("Period", per1@.Data + per2@.Data,
   	year = per1@year + per2@year,
@@ -38,19 +36,19 @@ add_period_to_period <- function(per2, per1){
 
 add_period_to_date <- function(per, date){
   lt <- as.POSIXlt(date)
-  
-  # add months and years with no backwards rollover
+
+  ## add months and years with no backwards rollover
   ms <- month(per) + year(per) * 12
   month(per) <- 0
   year(per) <- 0
   lt <- add_months(lt, ms)
   
-	new <- update(lt, 
-			days = mday(lt) + per@day,
-			hours = hour(lt) + per@hour,
-			minutes = minute(lt) + per@minute,
-			seconds = second(lt) + per@.Data
-			)
+  new <- update(lt, 
+                days = mday(lt) + per@day,
+                hours = hour(lt) + per@hour,
+                minutes = minute(lt) + per@minute,
+                seconds = second(lt) + per@.Data)
+  
 	if (is.Date(date) && sum(new$sec, new$min, new$hour, na.rm = TRUE) != 0)
 		return(new)	
 	
@@ -62,7 +60,6 @@ add_months <- function(mt, mos) {
   
   ndays <- as.numeric(format.POSIXlt(mt, "%d", usetz = FALSE))
   mt$mon[mt$mday != ndays] <- NA
-  
   mt
 }
 
@@ -77,7 +74,6 @@ add_number_to_period <- function(num, per){
 }
 
 
-
 #' @export
 setMethod("+", signature(e1 = "Duration", e2 = "Duration"),
 	function(e1, e2) add_duration_to_duration(e2, e1))
@@ -111,8 +107,6 @@ setMethod("+", signature(e1 = "Duration", e2 = "POSIXct"),
 #' @export	
 setMethod("+", signature(e1 = "Duration", e2 = "POSIXlt"), 
 	function(e1, e2) add_duration_to_date(e1, e2)) 
-	
-
 
 #' @export
 setMethod("+", signature(e1 = "Interval", e2 = "Duration"), function(e1, e2){ 
