@@ -15,9 +15,12 @@
 ##' truncated formats. For example \code{ymd} with \code{truncated = 2} will also
 ##' parse incomplete dates like \code{2012-06} and \code{2012}.
 ##'
-##' NOTE: \code{ymd} family of functions are based on \code{\link{strptime}}
-##' which currently correctly parses "\%y" format, but fails to parse "\%y-\%m"
-##' formats.
+##' NOTE: \code{ymd} family of functions are based on `parse_date_time` and thus
+##' directly drop to internal C parser for numeric months, but use R's
+##' `strptime` for alphabetic months. This implies that some of the `strptime`'s
+##' limitations are inherited by lubridate's parser. For example truncated
+##' formats (like %Y-%b) will not be parsed. Numeric truncated formats (like
+##' %Y-%m) are handled correctly by lubridate's C parser.
 ##'
 ##' As of version 1.3.0, lubridate's parse functions no longer return a
 ##' message that displays which format they used to parse their input. You can
@@ -63,7 +66,6 @@
 ##' ## formats are not in double digits might not be parsed correctly:
 ##' \dontrun{ymd("201002-01", "201002-1", "20102-1")
 ##' dmy("0312-2010", "312-2010")}
-##'
 ymd <- function(..., quiet = FALSE, tz = "UTC", locale = Sys.getlocale("LC_TIME"),  truncated = 0)
   .parse_xxx(..., orders = "ymd", quiet = quiet, tz = tz, locale = locale,  truncated = truncated)
 ydm <- function(..., quiet = FALSE, tz = "UTC", locale = Sys.getlocale("LC_TIME"),  truncated = 0)
@@ -216,16 +218,15 @@ ydm_h <- function(..., quiet = FALSE, tz = "UTC", locale = Sys.getlocale("LC_TIM
 ##' @seealso \code{\link{hms}, \link{hm}}
 ##' @keywords period
 ##' @examples
-##' x <- c("09:10", "09:02", "1:10")
-##' ms(x)
-##' # [1] 9 minutes and 10 seconds   9 minutes and 2 seconds   1 minute and 10 seconds
+##' ms(c("09:10", "09:02", "1:10"))
+##' ## [1] "9M 10S" "9M 2S"  "1M 10S"
 ##' ms("7 6")
-##' # [1] 7 minutes and 6 seconds
+##' ## [1] "7M 6S"
 ##' ms("6,5")
-##' # 6 minutes and 5 seconds
+##' ## [1] "6M 5S"
 ms <- function(..., quiet = FALSE) {
   out <- .parse_hms(..., order = "MS", quiet = quiet)
-  new_period(minute = out["M", ], second = out["S", ])
+  period(minute = out["M", ], second = out["S", ])
 }
 
 
@@ -243,16 +244,15 @@ ms <- function(..., quiet = FALSE) {
 ##' @seealso \code{\link{hms}, \link{ms}}
 ##' @keywords period
 ##' @examples
-##' x <- c("09:10", "09:02", "1:10")
-##' hm(x)
-##' # [1] 9 hours and 10 minutes   9 hours and 2 minutes   1 hour and 10 minutes
+##' hm(c("09:10", "09:02", "1:10"))
+##' ## [1] "9H 10M 0S" "9H 2M 0S"  "1H 10M 0S"
 ##' hm("7 6")
-##' # [1] 7 hours and 6 minutes
+##' ## [1] "7H 6M 0S"
 ##' hm("6,5")
-##' # [1] 6 hours and 5 minutes
+##' ## [1] "6H 5M 0S"
 hm <- function(..., quiet = FALSE) {
   out <- .parse_hms(..., order = "HM", quiet = quiet)
-  new_period(hour = out["H", ], minute = out["M", ])
+  period(hour = out["H", ], minute = out["M", ])
 }
 
 ##' Create a period with the specified hours, minutes, and seconds
@@ -271,19 +271,15 @@ hm <- function(..., quiet = FALSE) {
 ##' @useDynLib lubridate parse_hms
 ##' @examples
 ##'
-##' x <- c("09:10:01", "09:10:02", "09:10:03", "Collided at 9:20:04 pm")
+##' x <- c("09:10:01", "09:10:02", "09:10:03")
 ##' hms(x)
-##' # [1] 9 hours, 10 minutes and 1 second
-##' # [2] 9 hours, 10 minutes and 2 seconds
-##' # [3] 9 hours, 10 minutes and 3 seconds
+##' ## [1] "9H 10M 1S" "9H 10M 2S" "9H 10M 3S"
 ##'
-##' hms("7 6 5", "3-23---2", "2 : 23 : 33")
-##' ## [1] 7 hours, 6 minutes and 5 seconds
-##' ## [2] 3 hours, 23 minutes and 2 seconds
-##' ## [3] 2 hours, 23 minutes and 33 seconds
+##' hms("7 6 5", "3-23---2", "2 : 23 : 33", "Finished in 9 hours, 20 min and 4 seconds")
+##' ## [1] "7H 6M 5S" "3H 23M 2S" "2H 23M 33S" "9H 20M 4S"
 hms <- function(..., quiet = FALSE) {
   out <- .parse_hms(..., order = "HMS", quiet = quiet)
-  new_period(hour = out["H", ], minute = out["M", ], second = out["S", ])
+  period(hour = out["H", ], minute = out["M", ], second = out["S", ])
 }
 
 .parse_hms <- function(..., order, quiet = FALSE){
@@ -313,17 +309,17 @@ hms <- function(..., quiet = FALSE) {
 ##' When several format-orders are specified \code{parse_date_time} sorts the
 ##' supplied format-orders based on a training set and then applies them
 ##' recursively on the input vector.
-##'
+##' 
 ##' \code{parse_date_time}, and hence all the derived functions, such as
 ##' \code{ymd_hms}, \code{ymd} etc, will drop into \code{fast_strptime} instead
-##' of \code{strptime} whenever the trained from input data formats are all
+##' of \code{strptime} whenever the guesed from the input data formats are all
 ##' numeric.
-##'
-##' Here are all the formats recognized by lubridate. For numeric formats
-##' leading 0s are optional. As compared to \code{strptime}, some of the formats
-##' have been extended for efficiency reasons. They are marked with "*". Formats
-##' accepted by \code{parse_date_time2} and \code{fast_strptime} are marked with
-##' "!".
+##' 
+##' The list below contains formats recognized by lubridate. For numeric formats
+##' leading 0s are optional. In contrast to \code{strptime}, some of the formats
+##' have been extended for efficiency reasons. They are marked with "*". Fast
+##' perasers,  \code{parse_date_time2} and \code{fast_strptime}, currently
+##' accept only formats marked with "!".
 ##'
 ##' \describe{ \item{\code{a}}{Abbreviated weekday name in the current
 ##' locale. (Also matches full name)}
@@ -338,25 +334,25 @@ hms <- function(..., quiet = FALSE) {
 ##' 
 ##' \item{\code{B}}{Full month name in the current locale.  (Also matches abbreviated name.)}
 ##'
-##' \item{\code{d!}}{Day of the month as decimal number (01--31 or 0--31)}
+##' \item{\code{d}!}{Day of the month as decimal number (01--31 or 0--31)}
 ##' 
-##' \item{\code{H!}}{Hours as decimal number (00--24 or 0--24).}
+##' \item{\code{H}!}{Hours as decimal number (00--24 or 0--24).}
 ##' 
-##' \item{\code{I}}{Hours as decimal number (01--12 or 0--12).}
+##' \item{\code{I}}{Hours as decimal number (01--12 or 1--12).}
 ##' 
 ##' \item{\code{j}}{Day of year as decimal number (001--366 or 1--366).}
 ##' 
-##' \item{\code{m*!}}{Month as decimal number (01--12 or 1--12). For
+##' \item{\code{m}!*}{Month as decimal number (01--12 or 1--12). For
 ##' \code{parse_date_time}, also matches abbreviated and full months names as
 ##' \code{b} and \code{B} formats.}
 ##' 
-##' \item{\code{M!}}{Minute as decimal number (00--59 or 0--59).}
+##' \item{\code{M}!}{Minute as decimal number (00--59 or 0--59).}
 ##' 
 ##' \item{\code{p}}{AM/PM indicator in the locale.  Used in
 ##'                   conjunction with \code{I} and \bold{not} with \code{H}.  An
 ##'                   empty string in some locales.}
 ##' 
-##' \item{\code{S!}}{Second as decimal number (00--61 or 0--61), allowing for up
+##' \item{\code{S}!}{Second as decimal number (00--61 or 0--61), allowing for up
 ##' to two leap-seconds (but POSIX-compliant implementations will ignore leap
 ##' seconds).}
 ##'
@@ -372,23 +368,23 @@ hms <- function(..., quiet = FALSE) {
 ##' Monday as the first day of week (and typically with the first Monday of the
 ##' year as day 1 of week 1).  The UK convention.}
 ##'
-##' \item{\code{y*!}}{Year without century (00--99 or 0--99).  In
+##' \item{\code{y}!*}{Year without century (00--99 or 0--99).  In
 ##' \code{parse_date_time} also matches year with century (Y format).}
 ##' 
-##' \item{\code{Y!}}{Year with century.}
+##' \item{\code{Y}!}{Year with century.}
 ##'
-##' \item{\code{z*!}}{ISO8601 signed offset in hours and minutes from UTC. For
+##' \item{\code{z}!*}{ISO8601 signed offset in hours and minutes from UTC. For
 ##' example \code{-0800}, \code{-08:00} or \code{-08}, all represent 8 hours
 ##' behind UTC. This format also matches the Z (Zulu) UTC indicator. Because
-##' strptime doesn't fully support ISO8601, lubridate represents this format
-##' internally as an union of 4 different orders: Ou (Z), Oz (-0800), OO
-##' (-08:00) and Oo (-08). You can use this formats as any other but it is
-##' rarely necessary. \code{parse_date_time2} and \code{fast_strptime} support
-##' all of the timezone formats.}
+##' strptime doesn't fully support ISO8601 this format is implemented as an
+##' union of 4 orders: Ou (Z), Oz (-0800), OO (-08:00) and Oo (-08). You can use
+##' these four orders as any other but it is rarely
+##' necessary. \code{parse_date_time2} and \code{fast_strptime} support all of
+##' the timezone formats.}
 ##' 
-##' \item{\code{r*}}{Matches \code{Ip} and \code{H} orders.}
-##' \item{\code{R*}}{Matches \code{HM} and\code{IMp} orders.}
-##' \item{\code{T*}}{Matches \code{IMSp}, \code{HMS}, and \code{HMOS} orders.}
+##' \item{\code{r}*}{Matches \code{Ip} and \code{H} orders.}
+##' \item{\code{R}*}{Matches \code{HM} and\code{IMp} orders.}
+##' \item{\code{T}*}{Matches \code{IMSp}, \code{HMS}, and \code{HMOS} orders.}
 ##' }
 ##'
 ##' @export parse_date_time
@@ -426,47 +422,69 @@ hms <- function(..., quiet = FALSE) {
 ##' default selection method fails to select the formats in the right order. By
 ##' default the formats with most formating tockens (\%) are selected and \%Y
 ##' counts as 2.5 tockens (so that it has a priority over \%y\%m). Se examples.
+##' @param exact logical. If \code{TRUE}, \code{orders} parameter is interpreted
+##'   as an exact \code{strptime} format and no trainign or guessing are
+##'   performed.
 ##' @return a vector of POSIXct date-time objects
 ##' @seealso \code{strptime}, \code{\link{ymd}}, \code{\link{ymd_hms}}
 ##' @keywords chron
+##' @note \code{parse_date_time} (and the derivatives \code{ymb}, \code{ymd_hms}
+##' etc) rely on a sparse guesser that takes at most 501 elements from the
+##' supplied character vector in order to identify appropriate formats from the
+##' supplied orders. If you get the error \code{All formats failed to parse} and
+##' you are confident that your vector contains valid dates, you should use
+##' functions that don't perform format guessing (\code{strptime},
+##' \code{fast_strptime} and \code{parse_date_time2}).
 ##' @examples
+##'
+##' ## ** orders are much easier to write **
 ##' x <- c("09-01-01", "09-01-02", "09-01-03")
 ##' parse_date_time(x, "ymd")
+##' parse_date_time(x, "y m d")
 ##' parse_date_time(x, "%y%m%d")
-##' parse_date_time(x, "%y %m %d")
 ##' #  "2009-01-01 UTC" "2009-01-02 UTC" "2009-01-03 UTC"
 ##'
-##' ## ** heterogenuous formats **
+##' ## ** heterogenuous date-times **
 ##' x <- c("09-01-01", "090102", "09-01 03", "09-01-03 12:02")
-##' parse_date_time(x, c("%y%m%d", "%y%m%d %H%M"))
+##' parse_date_time(x, c("ymd", "ymd HM"))
 ##'
-##' ## different ymd orders:
+##' ## ** different ymd orders **
 ##' x <- c("2009-01-01", "02022010", "02-02-2010")
-##' parse_date_time(x, c("%d%m%Y", "ymd"))
+##' parse_date_time(x, c("dmY", "ymd"))
 ##' ##  "2009-01-01 UTC" "2010-02-02 UTC" "2010-02-02 UTC"
 ##'
 ##' ## ** truncated time-dates **
 ##' x <- c("2011-12-31 12:59:59", "2010-01-01 12:11", "2010-01-01 12", "2010-01-01")
-##' parse_date_time(x, "%Y%m%d %H%M%S", truncated = 3)
+##' parse_date_time(x, "Ymd HMS", truncated = 3)
 ##' parse_date_time(x, "ymd_hms", truncated = 3)
 ##' ## [1] "2011-12-31 12:59:59 UTC" "2010-01-01 12:11:00 UTC"
 ##' ## [3] "2010-01-01 12:00:00 UTC" "2010-01-01 00:00:00 UTC"
 ##'
+##' ## ** specifying exact formats and avoiding training and guessing **
+##' parse_date_time(x, c("%m-%d-%y", "%m%d%y", "%m-%d-%y %H:%M"), exact = TRUE)
+##' ## [1] "2001-09-01 00:00:00 UTC" "2002-09-01 00:00:00 UTC" NA "2003-09-01 12:02:00 UT
+##' parse_date_time(c('12/17/1996 04:00:00','4/18/1950 0130'),
+##'                 c('%m/%d/%Y %I:%M:%S','%m/%d/%Y %H%M'), exact = TRUE)
+##' ## [1] "1996-12-17 04:00:00 UTC" "1950-04-18 01:30:00 UTC"
+##' 
 ##' ## ** fast parsing **
 ##' \dontrun{
 ##'   options(digits.secs = 3)
 ##'   ## random times between 1400 and 3000
-##'   tt <- as.character(.POSIXct(runif(1e6, -17987443200, 32503680000)))
+##'   tt <- as.character(.POSIXct(runif(1000, -17987443200, 32503680000)))
+##'   tt <- rep.int(tt, 1000)
+##' 
 ##'   system.time(out <- as.POSIXct(tt, tz = "UTC"))
-##'   system.time(out1 <- ymd_hms(tt)) ## format learning overhead
+##'   system.time(out1 <- ymd_hms(tt)) # constant overhead on long vectors
 ##'   system.time(out2 <- parse_date_time2(tt, "YmdHMOS"))
 ##'   system.time(out3 <- fast_strptime(tt, "%Y-%m-%d %H:%M:%OS"))
+##' 
 ##'   all.equal(out, out1)
 ##'   all.equal(out, out2)
 ##'   all.equal(out, out3)
 ##' }
 ##' 
-##' ## ** how to use select_formats **
+##' ## ** how to use `select_formats` argument **
 ##' ## By default %Y has precedence:
 ##' parse_date_time(c("27-09-13", "27-09-2013"), "dmy")
 ##' ## [1] "13-09-27 UTC"   "2013-09-27 UTC"
@@ -481,7 +499,8 @@ hms <- function(..., quiet = FALSE) {
 ##' parse_date_time(c("27-09-13", "27-09-2013"), "dmy", select_formats = my_select)
 ##' ## '[1] "2013-09-27 UTC" "2013-09-27 UTC"
 parse_date_time <- function(x, orders, tz = "UTC", truncated = 0, quiet = FALSE,
-  locale = Sys.getlocale("LC_TIME"), select_formats = .select_formats){
+                            locale = Sys.getlocale("LC_TIME"), select_formats = .select_formats,
+                            exact = FALSE){
 
   orig_locale <- Sys.getlocale("LC_TIME")
   Sys.setlocale("LC_TIME", locale)
@@ -492,13 +511,21 @@ parse_date_time <- function(x, orders, tz = "UTC", truncated = 0, quiet = FALSE,
     orders <- .add_truncated(orders, truncated)
 
   .local_parse <- function(x, first = FALSE){
-    train <- .get_train_set(x)
-    formats <- .best_formats(train, orders, locale = locale, select_formats)
+    formats <-
+      if(exact){
+        orders
+      } else {
+        train <- .get_train_set(x)
+        .best_formats(train, orders, locale = locale, select_formats)
+      }
     if( length(formats) > 0 ){
       out <- .parse_date_time(x, formats, tz = tz, quiet = quiet)
       new_na <- is.na(out)
-      if( any(new_na) )
-        out[new_na] <- .local_parse(x[new_na])
+      if( any(new_na) ){
+        x <- x[new_na]
+        if(length(x) < length(out)) # don't recur if failed for all
+          out[new_na] <- .local_parse(x)
+      }
       out
     }else{
       if ( first && !quiet) {
@@ -523,12 +550,12 @@ parse_date_time <- function(x, orders, tz = "UTC", truncated = 0, quiet = FALSE,
   out
 }
 
-
 ##' @rdname parse_date_time
 ##' @export parse_date_time2
-parse_date_time2 <- function(x, orders, tz = "UTC"){
+parse_date_time2 <- function(x, orders, tz = "UTC", exact = FALSE){
   if( length(orders) > 1) warning("Multiple orders supplied. Only the first one is used")
-  orders <- gsub("[^[:alpha:]]+", "", as.character(orders[[1]])) ## remove all separators
+  if(!exact)
+    orders <- gsub("[^[:alpha:]]+", "", as.character(orders[[1]])) ## remove all separators
   .POSIXct(.Call("parse_dt", x, orders, FALSE), tz = tz)
 }
 
@@ -544,9 +571,9 @@ fast_strptime <- function(x, format, tz = "UTC"){
   .POSIXct(.Call("parse_dt", x, format, TRUE), tz = tz)
 }
 
+
 ### INTERNAL
 .parse_date_time <- function(x, formats, tz, quiet){
-  ## recursive parsing
   out <- .strptime(x, formats[[1]], tz = tz, quiet = quiet)
   na <- is.na(out)
   newx <- x[na]
@@ -555,6 +582,7 @@ fast_strptime <- function(x, format, tz = "UTC"){
   if( !is.null(verbose) && verbose )
     message(" ", sum(!na) , " parsed with ", gsub("^@|@$", "", formats[[1]]))
 
+  ## recursive parsing
   if( length(formats) > 1 && length(newx) > 0 )
     out[na] <- .parse_date_time(newx, formats[-1], tz = tz, quiet = quiet)
 
@@ -565,7 +593,7 @@ fast_strptime <- function(x, format, tz = "UTC"){
 .strptime <- function(x, fmt, tz = "UTC", quiet = FALSE){
 
   ## Depending on fmt we might need to preprocess x.
-  ## ISO8601 and fasttime are the only cases so far.
+  ## ISO8601 and internal parser are the only cases so far.
   ## %Ou: "2013-04-16T04:59:59Z"
   ## %Oo: "2013-04-16T04:59:59+01"
   ## %Oz: "2013-04-16T04:59:59+0100"
@@ -585,7 +613,7 @@ fast_strptime <- function(x, format, tz = "UTC"){
           message("Date in ISO8601 format; converting timezone from UTC to \"", tz,  "\".")
         with_tz(fast_strptime(x, fmt, tz = "UTC"), tzone = tz)
       } else {
-        ## as compared to the parsing, this is so unbelievably slow  :(
+        ## force_tz is very slow :(
         force_tz(fast_strptime(x, fmt, tz = "UTC"), tzone = tz)
       }
     } else {
@@ -676,7 +704,7 @@ fast_strptime <- function(x, format, tz = "UTC"){
     if( rest  < 0 )
       orders <- c(orders, add, .add_truncated(xxx, abs(rest)))
     else
-      orders <- c(orders, add[1:truncated], sep = "")
+      orders <- c(orders, add[1:truncated])
   }
   dates <- unlist(lapply(list(...), .num_to_date), use.names = FALSE)
   parse_date_time(dates, orders, tz = tz, quiet = quiet, locale = locale)
@@ -693,7 +721,7 @@ fast_strptime <- function(x, format, tz = "UTC"){
   if (is.numeric(x)) {
     out <- rep.int(as.character(NA), length(x))
     nnas <- !is.na(x)
-    x <- as.character(x[nnas])
+    x <- format(x[nnas], scientific = FALSE, trim = TRUE)
     x <- paste(ifelse(nchar(x) %% 2 == 1, "0", ""), x, sep = "")
     out[nnas] <- x
     out
